package com.elinkthings.ailinksecrettooldemo;

import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.os.Build;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.os.SystemClock;
import android.util.Log;

import com.elinkthings.ailinksecretlib.AiLinkBleCheckUtil;
import com.elinkthings.ailinksecretlib.BleConfig;
import com.elinkthings.ailinksecretlib.CmdConfig;
import com.elinkthings.ailinksecretlib.MyBleDeviceUtils;

import java.util.Arrays;
import java.util.UUID;

import androidx.annotation.NonNull;

/**
 * xing<br>
 * 2021/4/9<br>
 * 解析数据类
 */
public class AnalyticalDataUtil {

    /**
     * 握手指令
     */
    private final static int HANDSHAKE_FAILURE = 1;
    private BluetoothGatt mBluetoothGatt;
    /**
     * 当前是否为握手状态
     */
    private boolean mHandshakeStatus = true;
    private String mMac;
    private OnAnalyticalListener mOnAnalyticalListener;
    private Handler mHandler = new Handler(Looper.getMainLooper()) {
        @Override
        public void handleMessage(@NonNull Message msg) {
            switch (msg.what) {

                case HANDSHAKE_FAILURE:
                    //握手失败,握手超时
                    if (mOnAnalyticalListener != null) {
                        mOnAnalyticalListener.onHandshake(mMac, false);
                    }
                    break;

            }
        }
    };

    public AnalyticalDataUtil(BluetoothGatt gatt, OnAnalyticalListener listener) {
        mOnAnalyticalListener = listener;
        mBluetoothGatt = gatt;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            mBluetoothGatt.requestConnectionPriority(BluetoothGatt.CONNECTION_PRIORITY_HIGH);
        }
        mMac = gatt.getDevice().getAddress();
        openAILinkNotify(gatt);

    }


    /**
     * 通知返回数据
     */
    public final void notifyData(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
        byte[] hex = characteristic.getValue();
        if (hex == null) {
            return;
        }
        UUID uuid = characteristic.getUuid();
        getAnalyticalData(gatt, uuid, hex);
    }


    private void getAnalyticalData(BluetoothGatt gatt, UUID uuid, byte[] hex) {
        final String mac = gatt.getDevice().getAddress();
        if (AiLinkBleCheckUtil.checkReturnDataFormat(hex)) {
            //数据合法
            if (mHandshakeStatus) {
                //当前为握手状态
                if (hex[0] == CmdConfig.SEND_BLE_START && hex[2] == CmdConfig.GET_HANDSHAKE) {
                    boolean isData = getHandshakeStatus(hex);
                    if (isData) {
                        //握手成功
                        mHandler.removeMessages(HANDSHAKE_FAILURE);
                        mHandler.postDelayed(new Runnable() {
                            @Override
                            public void run() {
                                mHandshakeStatus = false;
                                if (mOnAnalyticalListener != null) {
                                    mOnAnalyticalListener.onHandshake(mac, true);
                                }
                            }
                        }, 500);//需要延迟500,确保芯片第二次验证通过
                    } else {
                        if (mOnAnalyticalListener != null) {
                            mOnAnalyticalListener.onHandshake(mac, false);
                        }
                    }
                    mDataHandshake = null;
                } else {
                    //不是握手状态
                    if (hex[0] == CmdConfig.SEND_BLE_START && hex[2] == CmdConfig.SET_HANDSHAKE) {
                        //是握手数据
                        byte[] bleDataHandshake = AiLinkBleCheckUtil.returnHandshakeDataFormat(hex);
                        sendHandshakePwd(bleDataHandshake);
                    } else {
                        protocolNotifyData(mac, uuid, hex);
                    }
                }
            } else {
                //不是握手状态
                if (hex[0] == CmdConfig.SEND_BLE_START && hex[2] == CmdConfig.SET_HANDSHAKE) {
                    //是握手数据
                    byte[] bleDataHandshake = AiLinkBleCheckUtil.returnHandshakeDataFormat(hex);
                    sendHandshakePwd(bleDataHandshake);
                } else {
                    protocolNotifyData(mac, uuid, hex);
                }

            }
        } else {
            protocolNotifyData(mac, uuid, hex);
        }
    }


    /**
     * 符合数据格式的协议notify数据处理
     */
    private void protocolNotifyData(String mac, UUID uuid, byte[] hex) {
        if (hex == null) {
            return;
        }
        byte[] payload;
        if (hex.length > 0) {
            switch (hex[0]) {
                case CmdConfig.SEND_BLE_START:
                    payload = AiLinkBleCheckUtil.returnBleDataFormat(hex);
                    if (payload != null && payload.length >= 1) {
                        if (mOnAnalyticalListener != null) {
                            mOnAnalyticalListener.onAnalyticalData(mac, uuid, payload);
                        }
                    }
                    break;

                case CmdConfig.SEND_MCU_START:
                    byte[] bytes = mcuDecrypt(hex, mac);
                    if (bytes.length > 0) {
                        if (mOnAnalyticalListener != null) {
                            mOnAnalyticalListener.onAnalyticalData(mac, uuid, bytes);
                        }
                    }
                    break;
                default:
                    if (mOnAnalyticalListener != null) {
                        mOnAnalyticalListener.onAnalyticalData(mac, uuid, hex);
                    }
                    break;
            }
        }
    }


    /**
     * 设置通知AILink
     */
    private void openAILinkNotify(BluetoothGatt gatt) {
        setNotify(gatt, BleConfig.UUID_WRITE_NOTIFY_AILINK, BleConfig.UUID_NOTIFY_AILINK);
    }

    /**
     * 开启多个Notify,如果多个服务,可重复调用
     *
     * @param gatt       BluetoothGatt
     * @param uuidNotify uuidNotify
     */
    public void setNotify(BluetoothGatt gatt, UUID... uuidNotify) {
        for (UUID uuid : uuidNotify) {
            setOpenNotify(gatt, uuid);
        }
    }

    /**
     * 设置通知
     *
     * @param gatt       BluetoothGatt
     * @param uuidNotify uuidNotify
     */
    private void setOpenNotify(BluetoothGatt gatt, UUID uuidNotify) {
        BluetoothGattService mGattService = MyBleDeviceUtils.getService(gatt, BleConfig.UUID_SERVER_AILINK);
        if (mGattService != null) {
            BluetoothGattCharacteristic mCharacteristic = MyBleDeviceUtils.getServiceWrite(mGattService, uuidNotify);
            if (mCharacteristic != null) {
                mCharacteristic.setWriteType(BluetoothGattCharacteristic.WRITE_TYPE_DEFAULT);//部分手机Notify需要设置这个后才会生效
                gatt.setCharacteristicNotification(mCharacteristic, true);
                BluetoothGattDescriptor bluetoothGattDescriptor = mCharacteristic.getDescriptor(BleConfig.UUID_NOTIFY_DESCRIPTOR);
                if (bluetoothGattDescriptor != null) {
                    bluetoothGattDescriptor.setValue(BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE);
                    boolean sendOk = gatt.writeDescriptor(bluetoothGattDescriptor);
                    if (!sendOk) {
                        SystemClock.sleep(50);
                        sendOk = gatt.writeDescriptor(bluetoothGattDescriptor);
                    }
                }
                if (uuidNotify.toString().equals(BleConfig.UUID_WRITE_NOTIFY_AILINK.toString())) {
                    //AILink notify success
                    sendHandshake(gatt);
                }
            }
        }

    }


    //---------------------握手指令发送校验 start--------------
    private volatile byte[] mDataHandshake;

    /**
     * 明文握手
     */
    public void sendHandshake(final BluetoothGatt gatt) {
        if (mHandshakeStatus) {
            sendCmd(initHandshakeArr());
            mHandshakeStatus = true;
            mHandler.removeMessages(HANDSHAKE_FAILURE);
            mHandler.sendEmptyMessageDelayed(HANDSHAKE_FAILURE, 5000);

        } else {
            if (mOnAnalyticalListener != null) {
                mOnAnalyticalListener.onHandshake(gatt.getDevice().getAddress(), true);
            }
        }
    }

    /**
     * 密文握手
     */
    public void sendHandshakePwd(byte[] hex) {
        byte[] bytes = initHandshakeArrPwd(hex);
        if (bytes != null) {
            sendCmd(bytes);
        }
    }

    private synchronized void sendCmd(final byte[] bytes) {
        if (bytes != null) {
            mHandler.post(new Runnable() {
                @Override
                public void run() {
                    SystemClock.sleep(100);
                    BluetoothGattService mGattService = MyBleDeviceUtils.getService(mBluetoothGatt, BleConfig.UUID_SERVER_AILINK);
                    if (mGattService != null) {
                        BluetoothGattCharacteristic mCharacteristic = MyBleDeviceUtils.getServiceWrite(mGattService, BleConfig.UUID_WRITE_NOTIFY_AILINK);
                        if (mCharacteristic != null) {
                            mCharacteristic.setValue(bytes);
                            mCharacteristic.setWriteType(BluetoothGattCharacteristic.WRITE_TYPE_NO_RESPONSE);
                            boolean b = mBluetoothGatt.writeCharacteristic(mCharacteristic);
                            Log.i("AnalyticalDataUtil", "send:" + b);
                        }
                    }
                }
            });
        }
    }


    /**
     * @return APP握手指令
     */
    private byte[] initHandshakeArr() {
        mDataHandshake = AiLinkBleCheckUtil.getRandomKey(16);
        return AiLinkBleCheckUtil.sendHandshakeFormat(mDataHandshake, CmdConfig.SET_HANDSHAKE);
    }

    /**
     * @param hex 明文数据
     * @return APP握手指令密文
     */
    private byte[] initHandshakeArrPwd(byte[] hex) {
        //加密数据
        byte[] appDataHandshake = AiLinkBleCheckUtil.bleEncrypt(hex);
        if (appDataHandshake != null) {
            return AiLinkBleCheckUtil.sendHandshakeFormat(appDataHandshake, CmdConfig.GET_HANDSHAKE);
        } else {
            return null;
        }
    }

    /**
     * 校验握手数据是否正确
     */
    private boolean getHandshakeStatus(byte[] data) {
        if (mDataHandshake == null) {
            return false;
        }
        //得到加密后的Payload
        byte[] bleDataHandshake = AiLinkBleCheckUtil.returnHandshakeDataFormat(data);
        //加密自己发送的数据
        byte[] appDataHandshake = AiLinkBleCheckUtil.bleEncrypt(mDataHandshake);//加密后会改变传入的byte数据,如需要保留请先copy
        //校验两次加密是否一致
        return Arrays.equals(appDataHandshake, bleDataHandshake);
    }


    //---------------------握手指令发送校验 end-------------


    /**
     * 解密A7数据
     *
     * @param hex byte[]
     * @param mac mac
     * @return byte[]
     */
    private byte[] mcuDecrypt(byte[] hex, String mac) {
        byte[] data = AiLinkBleCheckUtil.returnMcuDataFormat(hex);
        if (data != null && data.length >= 1) {
            byte[] CID = new byte[]{hex[1], hex[2]};
            data = AiLinkBleCheckUtil.mcuEncrypt(CID, data, mac);
        }
        return data;
    }


    /**
     * 加密A7数据
     *
     * @param cid     cid
     * @param mac     mac
     * @param payload byte[]
     * @return byte[]
     */
    private byte[] mcuEncrypt(int cid, String mac, byte[] payload) {
        byte[] data = null;
        if (payload != null && payload.length >= 1) {
            byte[] CID = new byte[2];
            CID[0] = (byte) ((cid >> 8) & 0xff);
            CID[1] = (byte) (cid);
            data = AiLinkBleCheckUtil.mcuEncrypt(CID, payload, mac);
        }
        return data;
    }


    /**
     * 将广播数据进行解密
     *
     * @param payload byte[]
     * @return byte[]
     */
    public byte[] decryptBroadcast(byte[] payload) {
        if (payload != null && payload.length >= 20) {
            byte[] cidB = new byte[1];
            byte[] vidB = new byte[1];
            byte[] pidB = new byte[1];
            int start = 0;
            System.arraycopy(payload, start, cidB, 0, cidB.length);
            start += 1;
            System.arraycopy(payload, start, vidB, 0, vidB.length);
            start += 1;
            System.arraycopy(payload, start, pidB, 0, pidB.length);
            start += 1;
            //特殊广播,包含是否可以绑定的标志
            int cid = (cidB[0] & 0xff);
            int vid = (vidB[0] & 0xff);
            int pid = (pidB[0] & 0xff);
            byte sum = payload[9];
            byte[] decryptData = new byte[10];//根据协议取出需要解密的广播数据
            System.arraycopy(payload, 10, decryptData, 0, decryptData.length);
            byte newSum = cmdSum(decryptData);
            if (sum == newSum) {
                byte[] data = AiLinkBleCheckUtil.decryptBroadcast(cid, vid, pid, decryptData);
                return data;
            }
        }
        return payload;
    }


    /**
     * 校验累加,从1开始加
     */
    private static byte cmdSum(byte[] data) {
        byte sum = 0;
        for (byte datum : data) {
            sum += datum;
        }
        return sum;
    }

    public void close() {
        if (mBluetoothGatt != null) {
            mBluetoothGatt.disconnect();
            mBluetoothGatt.close();
            mBluetoothGatt = null;
        }
    }

}
